package dev.kske.chess.ui;

import java.awt.EventQueue;
import java.awt.Toolkit;
import java.awt.dnd.DropTarget;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;

import dev.kske.chess.board.Board;
import dev.kske.chess.board.FENString;
import dev.kske.chess.exception.ChessException;
import dev.kske.chess.game.Game;
import dev.kske.chess.pgn.PGNDatabase;
import dev.kske.chess.pgn.PGNGame;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>MainWindow.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * 
 * @since Chess v0.1-alpha
 * @author Kai S. K. Engelbart
 */
public class MainWindow extends JFrame {

	private static final long serialVersionUID = -3100939302567978977L;

	private JTabbedPane tabbedPane;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(() -> {
			try {
				new MainWindow();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		});
	}

	/**
	 * Create the application.
	 */
	public MainWindow() {
		super("Chess by Kai S. K. Engelbart");
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		// Configure frame
		setResizable(false);
		setBounds(100, 100, 494, 565);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/pieces/queen_white.png")));

		// Add frame content
		tabbedPane = new JTabbedPane();
		getContentPane().add(tabbedPane);

		setJMenuBar(new MenuBar(this));
		new DropTarget(this, new GameDropTarget(this));

		// Update position and dimensions
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
	}

	/**
	 * @return The currently selected {@link GamePane} component
	 */
	public GamePane getSelectedGamePane() { return (GamePane) tabbedPane.getSelectedComponent(); }

	/**
	 * Creates a new {@link GamePane}, adds it to the tabbed pane and opens it.
	 * The new tab has the title {@code Game n} where {@code n} is its number.
	 * 
	 * @return The new {@link GamePane}
	 */
	public GamePane addGamePane() { return addGamePane("Game " + (tabbedPane.getComponentCount() + 1)); }

	/**
	 * Creates a new {@link GamePane}, adds it to the tabbed pane and opens it.
	 * 
	 * @param title The title of the {@link GamePane}
	 * @return The new {@link GamePane}
	 */
	public GamePane addGamePane(String title) {
		GamePane gamePane = new GamePane();
		tabbedPane.add(title, gamePane);
		tabbedPane.setSelectedIndex(tabbedPane.getComponentCount() - 1);
		return gamePane;
	}

	public GamePane addGamePane(String title, Board board) {
		GamePane gamePane = addGamePane(title);
		DialogUtil.showGameConfigurationDialog(this,
				(whiteName, blackName) -> {
					Game game = new Game(gamePane.getBoardPane(), whiteName, blackName, board);
					gamePane.setGame(game);
					game.start();
				});
		return gamePane;
	}

	/**
	 * Removes a {@link GamePane} form the tabbed pane.
	 * 
	 * @param index The index of the {@link GamePane} to remove
	 */
	public void removeGamePane(int index) { tabbedPane.remove(index); }

	/**
	 * Loads a game file (FEN or PGN) and adds it to a new {@link GamePane}.
	 * 
	 * @param files the files to load the game from
	 */
	public void loadFiles(List<File> files) {
		files.forEach(file -> {
			final String	name		= file.getName().substring(0, file.getName().lastIndexOf('.'));
			final String	extension	= file.getName().substring(file.getName().lastIndexOf('.')).toLowerCase();
			try {
				Board board;
				switch (extension) {
					case ".fen":
						board = new FENString(new String(Files.readAllBytes(file.toPath()), StandardCharsets.UTF_8)).getBoard();
						break;
					case ".pgn":
						PGNDatabase pgnDB = new PGNDatabase();
						pgnDB.load(file);
						if (pgnDB.getGames().size() > 0) {
							String[] gameNames = new String[pgnDB.getGames().size()];
							for (int i = 0; i < gameNames.length; i++) {
								final PGNGame game = pgnDB.getGames().get(i);
								gameNames[i] = String.format("%s vs %s: %s", game.getTag("White"), game.getTag("Black"), game.getTag("Result"));
							}
							JComboBox<String> comboBox = new JComboBox<>(gameNames);
							JOptionPane.showInputDialog(this, comboBox, "Select a game", JOptionPane.QUESTION_MESSAGE);
							board = pgnDB.getGames().get(comboBox.getSelectedIndex()).getBoard();
						} else throw new ChessException("The PGN database '" + name + "' is empty!");
						break;
					default:
						throw new ChessException("The file extension '" + extension + "' is not supported!");
				}
				addGamePane(name, board);
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(this,
						"Failed to load the file " + file.getName() + ": " + e.toString(),
						"File loading error",
						JOptionPane.ERROR_MESSAGE);
			}
		});
	}
}
